package java.nio.channels;

import java.nio.MappedByteBuffer;
import java.nio.ByteBuffer;
import java.util.Collections;
import java.util.HashSet;
import java.io.IOException;
import java.nio.file.OpenOption;
import java.util.Set;
import java.nio.file.Path;
import java.nio.file.attribute.FileAttribute;
import java.nio.channels.spi.AbstractInterruptibleChannel;

public abstract class FileChannel extends AbstractInterruptibleChannel implements SeekableByteChannel, GatheringByteChannel, ScatteringByteChannel
{
    private static final FileAttribute<?>[] NO_ATTRIBUTES;
    
    protected FileChannel() {
    }
    
    public static FileChannel open(Path path, Set<? extends OpenOption> options, FileAttribute<?>... attrs) throws IOException {
        return path.getFileSystem().provider().newFileChannel(path, options, attrs);
    }
    
    public static FileChannel open(Path path, OpenOption... elements) throws IOException {
        HashSet c = new HashSet(elements.length);
        Collections.addAll(c, elements);
        return open(path, c, FileChannel.NO_ATTRIBUTES);
    }
    
    @Override
    public abstract int read(ByteBuffer p0) throws IOException;
    
    @Override
    public abstract long read(ByteBuffer[] p0, int p1, int p2) throws IOException;
    
    @Override
    public final long read(ByteBuffer[] array) throws IOException {
        return this.read(array, 0, array.length);
    }
    
    @Override
    public abstract int write(ByteBuffer p0) throws IOException;
    
    @Override
    public abstract long write(ByteBuffer[] p0, int p1, int p2) throws IOException;
    
    @Override
    public final long write(ByteBuffer[] array) throws IOException {
        return this.write(array, 0, array.length);
    }
    
    @Override
    public abstract long position() throws IOException;
    
    @Override
    public abstract FileChannel position(long p0) throws IOException;
    
    @Override
    public abstract long size() throws IOException;
    
    @Override
    public abstract FileChannel truncate(long p0) throws IOException;
    
    public abstract void force(boolean p0) throws IOException;
    
    public abstract long transferTo(long p0, long p1, WritableByteChannel p2) throws IOException;
    
    public abstract long transferFrom(ReadableByteChannel p0, long p1, long p2) throws IOException;
    
    public abstract int read(ByteBuffer p0, long p1) throws IOException;
    
    public abstract int write(ByteBuffer p0, long p1) throws IOException;
    
    public abstract MappedByteBuffer map(FileChannel.MapMode p0, long p1, long p2) throws IOException;
    
    public abstract FileLock lock(long p0, long p1, boolean p2) throws IOException;
    
    public final FileLock lock() throws IOException {
        return this.lock(0L, Long.MAX_VALUE, false);
    }
    
    public abstract FileLock tryLock(long p0, long p1, boolean p2) throws IOException;
    
    public final FileLock tryLock() throws IOException {
        return this.tryLock(0L, Long.MAX_VALUE, false);
    }
    
    static {
        NO_ATTRIBUTES = new FileAttribute[0];
    }
    
    public static class MapMode
    {
        public static final FileChannel.MapMode READ_ONLY;
        public static final FileChannel.MapMode READ_WRITE;
        public static final FileChannel.MapMode PRIVATE;
        private final String name;
        
        private MapMode(String name) {
            this.name = name;
        }
        
        @Override
        public String toString() {
            return this.name;
        }
        
        static {
            READ_ONLY = new FileChannel.MapMode("READ_ONLY");
            READ_WRITE = new FileChannel.MapMode("READ_WRITE");
            PRIVATE = new FileChannel.MapMode("PRIVATE");
        }
    }
}
